import os

import requests
from requests.cookies import RequestsCookieJar

from dfiber.core.dfiber_exceptions import DfiberValueError, \
    DfiberAuthenticationException
from dfiber.core.models.constants import CosmosAuthEndpoint


class CosmosClient:
    """
    This class represents a client that interacts with Cosmos API for authentication

    Attributes:
        _endpoint (str): Dfiber Cosmos API endpoint.
    """

    def __init__(self):
        """
        Initialize CosmosClient with Dfiber Cosmos API endpoint.

        Args:
            None

        Raises:
            DfiberValueError: If DVIEW_COSMOS_ENDPOINT environment variable is not set.
        """

        self._endpoint = os.getenv('DVIEW_COSMOS_ENDPOINT')
        if not self._endpoint:
            raise DfiberValueError(error_type="CONFIGURATION_ERROR",
                                   name="MISSING_ENDPOINT",
                                   detail="Missing DVIEW_COSMOS_ENDPOINT environment variable.")

    def authenticate_with_email_password(self, email: str, password: str):
        """
        Authenticate with email and password.

        Args:
            email (string): The email used to authenticate with Dfiber.
            password (string):  The password associated with the Dfiber account.

        Returns:
            string: email.

        Raises:
            DfiberAuthenticationException: If the authentication fails.
        """
        response = requests.post(
            f"{self._endpoint}{CosmosAuthEndpoint.AUTH_ENDPOINT_EMAIL_LOGIN}",
            json={"email": email, "pass": password})
        if response.status_code == 200:
            return self.authenticate_with_cookie(response.cookies)
        else:
            raise DfiberAuthenticationException(
                error_type='USER_AUTHENTICATION_FAILED',
                name='DFIBER_EMAIL_PASSWORD_AUTH_FAILED',
                detail=f"Failed to authenticate with email-password-based authentication: {response.text}")

    def authenticate_with_api_key(self, api_key: str):
        """
        Authenticate with API key.

        Args:
            api_key (string): The API key used to authenticate with Dfiber.

        Returns:
            string: email.

        Raises:
            DfiberAuthenticationException: If the authentication fails.
        """
        response = requests.get(
            f"{self._endpoint}{CosmosAuthEndpoint.AUTH_ENDPOINT_API_KEY}",
            params={'key': api_key})
        if response.status_code == 200:
            email = response.json().get('email')
            return email
        else:
            raise DfiberAuthenticationException(
                error_type='USER_AUTHENTICATION_FAILED',
                name='DFIBER_API_KEY_AUTH_FAILED',
                detail=f"Failed to authenticate with API key-based authentication: {response.text}")

    def authenticate_with_cookie(self, cookies: dict | RequestsCookieJar):
        """
        Authenticate with cookie.

        Args:
            cookies (dict/RequestsCookieJar): The cookies used to authenticate with Dfiber.

        Returns:
            string: email.

        Raises:
            DfiberAuthenticationException: If the authentication fails.
        """
        response = requests.get(
            f"{self._endpoint}{CosmosAuthEndpoint.AUTH_ENDPOINT_COOKIE}",
            cookies=cookies)
        if response.status_code == 200:
            email = response.json().get('email')
            return email
        else:
            raise DfiberAuthenticationException(
                error_type='USER_AUTHENTICATION_FAILED',
                name='DFIBER_COOKIE_AUTH_FAILED',
                detail=f"Failed to authenticate with cookie-based authentication: {response.text}")
